from bs4 import BeautifulSoup
import requests
import re


class DiscogsScraper:
    def __init__(self, url, d):
        headers = {"User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_1)"}
        req = requests.get(url, headers=headers)
        soup = BeautifulSoup(req.text, "html.parser")
        self.soup = soup
        self.release_id = re.findall(r"\d+", soup.select("a.release-page")[0]["href"])[
            0
        ]
        self.d = d

    @property
    def media_id(self) -> str:
        return self.release_id

    @property
    def media_condition(self) -> str:
        media_html = self.soup.select("strong:-soup-contains('Media:')")[
            0
        ].find_next_sibling("span")
        media_text = re.split(r"\W+", media_html.text.strip())
        return f"{media_text[0]} ({media_text[1]})"

    @property
    def sleeve_condition(self) -> str:
        if not self.soup.select("strong:-soup-contains('Sleeve:')"):
            return "N/A"
        return self.soup.select("strong:-soup-contains('Sleeve:')")[
            0
        ].next_sibling.strip()

    @property
    def shipping_from(self) -> str:
        return self.soup.select("strong:-soup-contains('Item Ships From:')")[
            0
        ].next_sibling.strip()

    @property
    def suggestion_price(self) -> str:
        _info = self.d.release(self.release_id)
        try:
            value = _info.price_suggestions.mint.value
            currency = _info.price_suggestions.mint.currency
            return "%s %s" % (round(value, 2), currency)
        except:
            return "Unknow"


class DiscogerInfo:
    def __init__(self, url, d, release_id):
        self.url = url
        self.discogs = d

        if len(re.findall(r"\/master\/\d+", self.url)) == 1:
            self.media_type = "master"
        else:
            self.media_type = "release"

        self.release_id = release_id

    @property
    def release_info(self) -> dict:
        release_info = dict()
        if self.media_type == "master":
            master_release_info = self.discogs.master(self.release_id)
            all_info = self.discogs.release(master_release_info.main_release.id)
        else:
            all_info = self.discogs.release(self.release_id)
        release_info["release_id"] = self.release_id
        release_info["artist"] = all_info.artists[0].name
        release_info["title"] = all_info.title
        release_info["url"] = self.url
        release_info["type"] = self.media_type
        release_info["image"] = all_info.images[0]["uri"]
        release_info["last_sell"] = dict()
        return release_info



import feedparser
import re
import logging
from discoger import scrap


class CustomFormatter(logging.Formatter):
    """Logging Formatter to add colors"""

    OKGREEN = "\033[92m"
    WARNING = "\033[93m"
    FAIL = "\033[91m"
    ENDC = "\033[0m"
    fmt_nok = "%(asctime)s %(levelname)s (%(filename)s:%(lineno)d) - %(message)s"
    fmt_ok = "%(asctime)s %(levelname)s - %(message)s"

    FORMATS = {
        logging.INFO: OKGREEN + fmt_ok + ENDC,
        logging.WARNING: WARNING + fmt_nok + ENDC,
        logging.ERROR: FAIL + fmt_nok + ENDC,
    }

    def format(self, record):
        log_fmt = self.FORMATS.get(record.levelno)
        formatter = logging.Formatter(log_fmt)
        return formatter.format(record)


def check_sales(self, release_id, type_sell):
    self.discogs_url = "https://www.discogs.com"
    data_last_sell = dict()
    if type_sell == "master":
        url = f"{self.discogs_url}/sell/mplistrss?output=rss&master_id={release_id}&ev=mb&format=Vinyl"
    else:
        url = f"{self.discogs_url}/sell/mplistrss?output=rss&release_id={release_id}"
    feed = feedparser.parse(url)
    try:
        entry = feed.entries[-1]
        data_last_sell["id"] = re.findall(r"\d+", entry["link"])[0]
        data_last_sell["date"] = entry["updated"]
        data_last_sell["url"] = entry["link"]
        data_last_sell["price"] = re.findall(
            r"... \d?\d?\d\d.\d\d", entry["summary_detail"]["value"]
        )[0]
        if (
            self.disable_unofficial
            and len(re.findall("Unofficial", entry["title"])) > 0
        ):
            return None
        else:
            return data_last_sell
    except Exception as e:
        logging.debug("%s: for %s item" % (e, release_id))
        return None


def scrap_data(self, chat_id, db):
    chat_id = db.get("chat_id")
    for i in range(len(db["release_list"])):
        item = db.search("release_list[%s]" % (str(i)))
        type_sell = item.get("type")
        if type_sell is None:
            type_sell = "release"
        data_last_sell = check_sales(self, item["release_id"], type_sell)
        if data_last_sell:
            if not item["last_sell"] or (
                item["last_sell"]["id"] != data_last_sell["id"]
                and item["last_sell"]["date"] < data_last_sell["date"]
            ):
                parse = scrap.DiscogsScraper(data_last_sell["url"], self.d)
                logging.info("New item for %s - %s" % (item["artist"], item["title"]))
                text = """
**New release for:**
%s - %s
Date: %s
Price: %s
Recommanded price: %s
Media: %s
Sleeve: %s
Shipping from: %s
%s
                """ % (
                    item["artist"],
                    item["title"],
                    data_last_sell["date"],
                    data_last_sell["price"],
                    parse.suggestion_price,
                    parse.media_condition,
                    parse.sleeve_condition,
                    parse.shipping_from,
                    data_last_sell["url"],
                )
                if "image" in item:
                    self.bot.send_photo(
                        chat_id,
                        photo=item["image"],
                        parse_mode="markdown",
                        caption=text,
                    )
                else:
                    self.bot.send_message(
                        chat_id,
                        text,
                        parse_mode="markdown",
                        disable_web_page_preview=True,
                    )
                db["release_list"][i]["last_sell"] = data_last_sell
            else:
                logging.info(
                    "Not new item for %s - %s"
                    % (
                        db["release_list"][i]["artist"],
                        db["release_list"][i]["title"],
                    )
                )
        else:
            logging.info(
                "Nothing available for %s - %s"
                % (
                    db["release_list"][i]["artist"],
                    db["release_list"][i]["title"],
                )
            )
    db.save()
